@@ -516,6 +516,11 @@ static MagickBooleanType IsBMP(const unsigned char *magick,const size_t length)
 %
 */
 
+static inline MagickBooleanType BMPOverflowCheck(size_t x,size_t y)
+{
+  return((y != 0) && (x > 4294967295UL/y) ? MagickTrue : MagickFalse);
+}
+
 static Image *ReadEmbedImage(const ImageInfo *image_info,Image *image,
   const char *magick,ExceptionInfo *exception)
 {
@@ -609,6 +614,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     bit,
     bytes_per_line,
+    extent,
     length;
 
   ssize_t
@@ -1111,12 +1117,18 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     if (bmp_info.compression == BI_RLE4)
       bmp_info.bits_per_pixel<<=1;
-    bytes_per_line=4*((image->columns*bmp_info.bits_per_pixel+31)/32);
-    length=(size_t) bytes_per_line*image->rows;
+		extent=image->columns*bmp_info.bits_per_pixel;
+		bytes_per_line=4*((extent+31)/32);
+    if (BMPOverflowCheck(bytes_per_line,image->rows) != MagickFalse)
+      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    length=bytes_per_line*image->rows;
     if ((MagickSizeType) (length/256) > blob_size)
       ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-    pixel_info=AcquireVirtualMemory(image->rows,
-      MagickMax(bytes_per_line,image->columns+1UL)*sizeof(*pixels));
+    extent=MagickMax(bytes_per_line,image->columns+1UL);
+    if ((BMPOverflowCheck(image->rows,extent) != MagickFalse) ||
+        (BMPOverflowCheck(extent,sizeof(*pixels)) != MagickFalse))
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    pixel_info=AcquireVirtualMemory(image->rows,extent*sizeof(*pixels));
     if (pixel_info == (MemoryInfo *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     pixels=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
@@ -1857,6 +1869,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
 
   size_t
     bytes_per_line,
+    extent,
     number_scenes,
     type;
 
@@ -2027,7 +2040,10 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
               }
           }
       }
-    bytes_per_line=4*((image->columns*bmp_info.bits_per_pixel+31)/32);
+    extent=image->columns*(size_t) bmp_info.bits_per_pixel;
+    bytes_per_line=4*((extent+31)/32);
+    if (BMPOverflowCheck(bytes_per_line,image->rows) != MagickFalse)
+      ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
     bmp_info.ba_offset=0;
     if (type > 3)
       profile=GetImageProfile(image,"icc");
@@ -2089,8 +2105,12 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
     /*
       Convert MIFF to BMP raster pixels.
     */
-    pixel_info=AcquireVirtualMemory(image->rows,MagickMax(bytes_per_line,
-      image->columns+256UL)*sizeof(*pixels));
+    extent=MagickMax(bytes_per_line,image->columns+1UL)*
+      ((bmp_info.bits_per_pixel+7)/8);
+    if ((BMPOverflowCheck(image->rows,extent) != MagickFalse) ||
+        (BMPOverflowCheck(extent,sizeof(*pixels)) != MagickFalse))
+      ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
+    pixel_info=AcquireVirtualMemory(image->rows,extent*sizeof(*pixels));
     if (pixel_info == (MemoryInfo *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     pixels=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
